// Copyright 2000 by Kevin Atkinson under the terms of the LGPL

#include <iomanip>

#include "as_config.h"

#include "config.hh"
#include "file_exceps.hh"
#include "bad_value.hh"

#define MODES "none,url,email,sgml,tex"

namespace aspell {
  // 
  // GlobalConf
  //
  void Config::read_in(const Config * override) 
  {
    if (override != 0)
      read_in(&const_cast<Config *>(override)->real_config());
    else
      read_in();
  }
  
  void Config::read_in(PspellConfigImpl * override) 
  {
    if (override != 0)
      real_config().merge(*override);

    const char * env = getenv("ASPELL_CONF");
    if (env != 0)       
      read_in_string(env);
    
    Config per;
    try {
      per.read_in_file(retrieve("per-conf-path"));
    } catch (CantReadFile) {}
    merge(per);

    
    try {
      read_in_file(retrieve("conf-path"));
    } catch (CantReadFile) {}
    
    merge(per);
    if (env != 0) 
      read_in_string(env);
    if (override != 0)
      real_config().merge(*override);
  }

  inline string get_mode_data(Config & c, const string & mode) {
    try {
      string temp = "fm-" + mode;
      return c.retrieve(temp.c_str());
    } catch (UnknownKey) {
      throw BadValue("mode", mode, "one of " MODES);
    }
  }

  class Config::NotifierImpl : public Config::Notifier
  {
  private:
    NotifierImpl();
    NotifierImpl(const NotifierImpl &);
    NotifierImpl & operator= (const NotifierImpl &);
  public:
    Config * config;
    NotifierImpl(Config * c) : config(c) {}

    void item_updated(const KeyInfo * ki, const char * value) {
      if (strcmp(ki->name, "mode") == 0) {

	string m = get_mode_data(*config, value);
	config->replace("rem-all-filter","");
	int begin = 0, end = 0;

	do {
	  
	  while (end < m.size() && m[end] != ',') ++end;
	  string fil = m.substr(begin, end - begin);
	  config->replace("add-filter", fil.c_str());
	  begin = end + 1;
	  end   = begin;
	  
	} while (end < m.size());

      }
    }
  };

#define CANT_CHANGE 1

  static const KeyInfo config_keys[] = {
    {"charset",  KeyInfoString, "iso8859-1",0}
    , {"conf",     KeyInfoString, "aspell.conf",  "main configuration file"             , {0, CANT_CHANGE}}
    , {"conf-dir", KeyInfoString, CONFIGDIR,      "location of main configuration file" ,{0, CANT_CHANGE}}
    , {"conf-path",     KeyInfoString, "<conf-dir/conf>",     0}
    , {"data-dir", KeyInfoString, DATADIR,        "location of language data files"     }
    , {"local-data-dir", KeyInfoString, DATADIR,        "location of local language data files"     }
    , {"dict-dir", KeyInfoString, DICTDIR,        "location of the main word list"      }
    , {"filter",   KeyInfoList  , "url",             "add or removes a filter"}
    , {"fm-email", KeyInfoList  , "url,email",                         0}
    , {"fm-none",  KeyInfoList  , "",                                  0}
    , {"fm-sgml",  KeyInfoList  , "url,sgml,SGML&<charset>/<charset>", 0}
    , {"fm-tex",   KeyInfoList  , "url,tex",                           0}
    , {"fm-url",   KeyInfoList  , "url",                               0}
    , {"home-dir", KeyInfoString, "<$HOME|./>",   "location for personal files" }
    , {"ignore",   KeyInfoInt   , "1",            "ignore words <= n chars"             }
    , {"ignore-accents" , KeyInfoBool, "false", "ignore accents when checking words"}
    , {"strip-accents" , KeyInfoBool, "false", "strip accents from word lists"}
    , {"ignore-case", KeyInfoBool  , "false",     "ignore case when checking words"}
    , {"ignore-repl", KeyInfoBool  , "false",     "ignore commands to store replacement pairs"}
    , {"lang",     KeyInfoString, "english",      "default language to use", {0, CANT_CHANGE}}
    , {"master",   KeyInfoString, "<lang>",       "main word list base name",           }
    , {"master-path",   KeyInfoString, "<dict-dir/master>",   0}
    , {"mode",     KeyInfoString, "url",             "filter mode = " MODES }
    , {"per-conf", KeyInfoString, ".aspell.conf", "personal configuration file",{0, CANT_CHANGE}}
    , {"per-conf-path", KeyInfoString, "<home-dir/per-conf>", 0}
    , {"personal", KeyInfoString, ".aspell.<lang>.pws",   "personal word list file name"}
    , {"personal-path", KeyInfoString, "<home-dir/personal>", 0}
    , {"repl",     KeyInfoString, ".aspell.<lang>.prepl", "replacements list file name" }
    , {"repl-path",     KeyInfoString, "<home-dir/repl>",     0}
    , {"run-together",        KeyInfoBool,  "false", "consider run-together words legal"}
    , {"run-together-limit",  KeyInfoInt,   "8", "maxium numbers that can be strung together"}
    , {"run-together-min",    KeyInfoInt,   "3", "minimal length of interior words"}
    , {"sug-mode",    KeyInfoString, "normal",    "suggestion mode = fast|normal|bad-spellers"}
    , {"run-together-specified", KeyInfoBool, "false", 0}
    , {"minimal-specified-component", KeyInfoInt, "255", 0}
    , {"extra-dicts", KeyInfoList, "", "extra dictionaries to use"}
  };
}


namespace afilter {
  typedef const autil::KeyInfo * ConstKeyInfoPtr;
  extern ConstKeyInfoPtr 
    email_options_begin, email_options_end,
    sgml_options_begin, sgml_options_end,
    tex_options_begin, tex_options_end;
}

namespace aspell {
  static const ConfigData::Module modules[] = {
    {"email", afilter::email_options_begin, afilter::email_options_end}
    , {"sgml",  afilter::sgml_options_begin,  afilter::sgml_options_end}
    , {"tex",   afilter::tex_options_begin,   afilter::tex_options_end}
  };
  
  Config::Config() 
    : ConfigData ("aspell",
		  config_keys, 
		  config_keys + sizeof(config_keys)/sizeof(KeyInfo))
    , notifier(new NotifierImpl(this))
  {
    set_modules(modules, modules+ sizeof(modules)/sizeof(ConfigData::Module));
    add_notifier(notifier);
  }

  Config::~Config() {
    delete notifier;
  }

  Config::Config(const Config & other)
    : ConfigData(other),
      notifier(new NotifierImpl(this))
  {
    replace_notifier(other.notifier, notifier);
  }

  Config & Config::operator= (const Config & other) 
  {
    ConfigData::operator=(other);
    replace_notifier(other.notifier, notifier);
    return *this;
  }
  
}
